#!/usr/bin/env python
from os import (
    chmod,
    fchmod,
    chown,
    fchown,
    makedirs,
    )
from os.path import abspath
from os.path import dirname
from os.path import exists
from pwd import getpwnam
from string import Template


DEPLOYMGR_TEMPLATE = 'templates/deploymgr.sh'
DEPLOYMGR_OUT = '/srv/deploymgr/charmworld'
CHARM_DIR = abspath(dirname(dirname(__file__)))


def prep_output_path(path):
    """Verify that we can write to the output location."""
    directory = dirname(path)
    if not exists(directory):
        makedirs(directory)
        chmod(directory, 0755)
        chown(directory, WEBOPS_UID, WEBOPS_GID)


def write_deploymgr():
    with open(DEPLOYMGR_TEMPLATE) as handle:
        template = Template(handle.read())
        rendered = template.substitute({
            'charm_dir': CHARM_DIR
        })
        with open(DEPLOYMGR_OUT, 'w') as output:
            fchown(output.fileno(), WEBOPS_UID, WEBOPS_GID)
            fchmod(output.fileno(), 0755)
            output.writelines(rendered)


if __name__ == "__main__":
    WEBOPS_UID, WEBOPS_GID = getpwnam('webops_deploy')[2:4]
    prep_output_path(DEPLOYMGR_OUT)
    write_deploymgr()
