#!/usr/bin/env python

from ConfigParser import ConfigParser
import json
import subprocess
import sys

from charmsupport.hookenv import related_units
from charmsupport.hookenv import relation_ids


def read_config(files):
    new_config = ConfigParser()
    new_config.read(files)
    return new_config


def write_new_config(mongo_url, default, override, target):
    new_config = read_config([default, override])
    new_config.set('app:main', 'mongo.url', mongo_url)
    with open(target, 'w') as output:
        new_config.write(output)


def get_relation_id_data(relation_id, unit):
    """Return a dictionary containing relation settings.

    :param relation_id: The relation id specifying the relation.
    :param unit: The name of the unit participating in the relation.
    """
    cmd = ['relation-get', '--format=json', '-r', relation_id, '-', unit]
    return json.loads(subprocess.check_output(cmd))


def get_mongo_url():
    """Use the 'database' relation settings to build a mongodb url.

    If mongodb units are present, return an empty string.
    """
    host_ports = []
    replset = None
    for relation_id in relation_ids('database'):
        for unit in related_units(relation_id):
            relation_data = get_relation_id_data(relation_id, unit)
            # Consider configuration incomplete until port is supplied.
            if relation_data is None or 'port' not in relation_data:
                continue
            if replset is None:
                replset = relation_data['replset']
            # All units should be members of the same replication set.
            elif replset != relation_data['replset']:
                raise AssertionError('DB instances are from different sets!')
            host_ports.append(
                '%(private-address)s:%(port)s' % relation_data)
    if len(host_ports) == 0:
        return ''
    return 'mongodb://%s/?replicaSet=%s' % (','.join(host_ports), replset)


class UsageError(Exception):
    """An error triggered by incorrect usage"""


def main():
    if len(sys.argv) < 4:
        raise UsageError('Usage: %s DEFAULTS OVERRIDES TARGET')
    parser = write_new_config(get_mongo_url(), default=sys.argv[1],
                              override=sys.argv[2], target=sys.argv[3])


if __name__ == '__main__':
    try:
        main()
    except UsageError as e:
        sys.stderr.write(str(e))
        sys.exit(1)
